package pl.net.linux.krzak.kchat.server;
/*****************************/
/* kChat v1.9		     */
/* krzak@linux.net.pl        */
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.util.*;
import java.io.*;
import pl.net.linux.krzak.kchat.util.kChatParse;

public class kChatSystem {

    private kChatParse parse = new kChatParse();
    public String logfile = new String("./chanlog.log");
    public String log = new String("no");
    public int logLevel = 1;
    public String rnicksFilename = "off";
    public String logoutFilename = "";
    public String passwdfile = "";
    public Hashtable moderpassHT = new Hashtable();
    public Hashtable moderatedHT  = new Hashtable();
    public Hashtable topicHT = new Hashtable();
    public Vector clientList = new Vector();
    public String lastError = "";
    public Hashtable moderatorNickHT = new Hashtable();
    public Hashtable guestNickHT = new Hashtable();
    public String topicDef = "kChat 2.0pre by Marcin Krzyzanowski <krzak@linux.net.pl>";
    public String TXT[] = new String[19];

/*******************************************************************************************/
/* answer for action send to server 							   */
/*******************************************************************************************/
    public String getAnswerFor(String ln, kChatServerThread th)
    {
    if ((ln == null) || (ln.equals(null)) || (ln.equals(""))) return("UNKNOWN COMMAND");
    /* *CLOSE */
    if (ln.equals("*CLOSE")) {
	quitFromSystem(th.login);
	return("BYE");
	}

    /* *WHOAMI */
    if (ln.equals("*WHOAMI")) return(th.clientS.getInetAddress().getHostAddress());

    /* *MODERATED */
    if (ln.equals("*MODERATED")) {
	Boolean bval = (Boolean)moderatedHT.get(th.channelName);
	if (bval.booleanValue())
	    return("*MODERATED|YES|"+(String)moderatorNickHT.get(th.channelName)+"|");
	    else
	    return("*MODERATED|NO|");
	}

    /* *LOGIN|SYSTEM|loginname|password|channel|color| */
    if (ln.startsWith("*LOGIN")) {
	String who = parse.getParam(ln,3);
	String password = parse.getParam(ln,4);
	String chantmp = parse.getParam(ln,5);
	String color = parse.getParam(ln,6);
	if ((parse.getParam(ln,2).equals("SYSTEM")) &&    // *LOGIN|SYSTEM|
	    (!who.equals("")) &&	            // loginname not empty
	    (loginToSystem(who,password,chantmp,color,th))) {     // login to system
	       return("*LOGIN|OK|"+th.login+"|"+logoutFilename+"|");
	    } else {
	       return("*LOGIN|NICK BUSY|");
	    }
	}

    /* *MSG|who|where|text|color| */
    if (ln.startsWith("*MSG"))
 {
        if (th.login.equals("")) return("*ERROR|LOGIN FIRST|");
	if (parse.getParam(ln,4).equals("")) return("*ERROR|UNKNOWN|");
	sendMsgTo(th.login,parse.getParam(ln,3),parse.getParam(ln,4),parse.getParam(ln,5),th.channelName);
        return("");
        }

    /* *CHECK */
    if (ln.startsWith("*CHECK")) {
	String chantmp = parse.getParam(ln,2);
	String out = "*LIST|";
	boolean empty = true;
	if (clientList.size()<=0) return("*LIST|NULL|");
	for (int i=0; i<clientList.size(); i++) {
	    kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	    if (tmpE.channelName.equals(chantmp)) {
			out = out + tmpE.login + "|";
			empty = false;
			}
	    }
	if (empty) return("*LIST|NULL|");
	return(out);
    }

    /* *MODERATOR|pass|guest| */
    if (ln.startsWith("*MODERATOR")) {
	if (parse.getParam(ln,2).equals("") || parse.getParam(ln,3).equals("")) return("");
	String typedPass  = parse.getParam(ln,2);

	    moderpassHT.clear();
	    try {
	    FileReader frpass = new FileReader(passwdfile);
	    BufferedReader frbpass = new BufferedReader(frpass);
	    String linepass = new String();
	    while (linepass!=null) {
		linepass = frbpass.readLine();
		if ((linepass!=null)&&(!linepass.startsWith("#"))&&(!linepass.equals(""))) {
		    String txttmp = linepass.substring(0,linepass.indexOf("=",0));
		    String valtmp = linepass.substring(linepass.indexOf("=",0)+1);
		    moderpassHT.put(txttmp,valtmp);
		    }
		}
	    }
	    catch (IOException e) { System.out.println("Check passfile !!!"); }


	// moderpass is taken everytime from configuration file
	if ((moderpassHT.get(th.channelName)!=null)&&(typedPass.equals((String)moderpassHT.get(th.channelName)))) {
		String typedGuest = parse.getParam(ln,3);
		moderatorNickHT.put(th.channelName,th.login);
		guestNickHT.put(th.channelName,typedGuest);
		moderatedHT.put(th.channelName,new Boolean(true));
		sayToAll("*MODERATED|YES|" + (String)moderatorNickHT.get(th.channelName) + "|",th.channelName);
		return("*YOUAREMODERATOR");
		}
	}
    /* *NOMODERATOR */
    if (ln.startsWith("*NOMODERATOR") && (th.login.equals((String)moderatorNickHT.get(th.channelName)))) {
	sayToAll("*MODERATED|NO|",th.channelName);
	moderatorNickHT.put(th.channelName,new String(""));
	guestNickHT.put(th.channelName,new String(""));
	moderatedHT.put(th.channelName,new Boolean(false));
	return("");
    }


    /* *KICK|nickname| */
    Boolean bval = (Boolean)moderatedHT.get(th.channelName);
    if (ln.startsWith("*KICK") && (bval.booleanValue()) && (th.login.equals((String)moderatorNickHT.get(th.channelName))) && (!parse.getParam(ln,2).equals(""))) {
	String tokick = parse.getParam(ln,2);
	for (int i=0;i<clientList.size();i++) {
	    kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	    if (tmpE.login.equals(tokick)) {
		tmpE.watek.pw.println("*KICK\r\n");
		tmpE.watek.pw.flush();
		}
	    }
	quitFromSystem(tokick);
	return("");
	}

    /* *PONG */
    if (ln.startsWith("*PONG")) {
	th.ping = 0;
	return("");
    }

    /* *LANG */
    if (ln.startsWith("*LANG")) {
	String out = "*LANG|";
	for (int i=0;i<TXT.length;i++) out = out + TXT[i] + "|";
	return(out);
    }

    /* *TOPIC|SET/GET|channel|text| */
    if (ln.startsWith("*TOPIC")) {
	String chantmp = parse.getParam(ln,3);
	if (parse.getParam(ln,2).equals("SET") && (th.login.equals((String)moderatorNickHT.get(th.channelName))))  {
	    topicHT.put(chantmp,parse.getParam(ln,4));
	    sayToAll("*TOPIC|GET|" + topicHT.get(chantmp) + "|",chantmp);
	    }

	if (!topicHT.containsKey(chantmp)) topicHT.put(chantmp,topicDef);

	sayToAll("*TOPIC|GET|" + topicHT.get(chantmp) + "|",chantmp);
	return("");
    }



    return("UNKNOWN COMMAND");
    }
/*******************************************************************************************/
/* write to logfile  							   */
/*******************************************************************************************/
    public void writeToLogfile(String text,String chan) {
        try {
    	    FileWriter fw = new FileWriter(logfile+"."+chan,true);
	    BufferedWriter fwb = new BufferedWriter(fw);
	    fwb.write(text);
	    fwb.newLine();
	    fwb.flush();
	    fwb.close();
	    } catch (IOException e) { e.printStackTrace(); }
    }

/*******************************************************************************************/
/* broadcast message  							   */
/*******************************************************************************************/
    public void sayToAll(String s,String chantmp) {
    kChatClass tmpE = null;

    // do ludkow z chantmp tylko
    for (int i = 0; i < clientList.size(); i++) {
	  tmpE = (kChatClass)clientList.elementAt(i);
	  if (tmpE.channelName.equals(chantmp)) {
             tmpE.watek.pw.println(s+"\r\n");
             tmpE.watek.pw.flush();
	     }
	 }

    if (log.equals("yes")) {
	Calendar now = Calendar.getInstance();
	String ampm = "AM";
	if (now.get(Calendar.AM_PM) == 1) ampm = "PM";
	String time = now.get(Calendar.DAY_OF_MONTH) + "/" + now.get(Calendar.MONTH) + "/" + now.get(Calendar.YEAR) + " " + now.get(Calendar.HOUR) +":"+ now.get(Calendar.MINUTE) + " " +ampm+" ";
        String st = new String(time);
	// log *MSG to file with logLevel >= 1
	if ((s.startsWith("*MSG")) && (logLevel>=1)) {
	 st = st + "<" + parse.getParam(s,2) + "> " + parse.getParam(s,4);
	 writeToLogfile(st,chantmp);
	}
        // log *QUIT to file with logLevel >= 2
	if ((s.startsWith("*QUIT")) && (logLevel>=2)) {
	 st = st + "QUIT " + parse.getParam(s,2);
	 writeToLogfile(st,chantmp);
	}
    }
}
/*******************************************************************************************/
/* LOGIN TO SYSTEM  should be fine Mon Oct 22 13:00:50 2000  				   */
/*******************************************************************************************/
    public boolean loginToSystem(String login, String password, String chantmp, String color, kChatServerThread th)
 {
    if ((login == null) || (login.equals(""))) return(false);
    login = login.replace(' ', '_');
    login = login.replace('|', '_');
    login = login.replace('<', '_');
    login = login.replace('>', '_');
    login = login.replace('~', '_');
    login = login.replace('\r', '_');
    login = login.replace('\n', '_');
    login = login.replace('\t', '_');
    if (login.length()>10) login = login.substring(0,10);

    Vector rnicksList = new Vector();
    Vector rnickspassList = new Vector();
    // reserverd list check
    if ((rnicksFilename!=null) && (!rnicksFilename.equals("")) && (!rnicksFilename.equals("off")))
	try {
    	    FileReader fr = new FileReader(rnicksFilename);
    	    BufferedReader frb = new BufferedReader(fr);
	    String line = new String();
	    while (line!=null) {
		line = frb.readLine();
		if ((line!=null)&&(!line.startsWith("#"))&&(!line.equals(""))) {
		    String rnick = line.substring(0,line.indexOf("=",0));
		    String rnickpass   = line.substring(line.indexOf("=",0)+1);
		    rnicksList.addElement(rnick);
		    rnickspassList.addElement(rnickpass);
		}
	    }
	} catch (IOException e) { System.out.println("Check reserved-nicks-list-file option ot its file !!!"); }


    // check for reserved nick
    for(int j=0;j<rnicksList.size();j++) {
        String rn = (String)rnicksList.elementAt(j);
        String rnpass = (String)rnickspassList.elementAt(j);
        if ((rn.equals(login))&&(!rnpass.equals(password))) return(false);
        }

    for (int i = 0; i < clientList.size(); i++) {
	kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(login))  return(false);
	}

    kChatClass tmpC = new kChatClass();
    tmpC.login = login;
    tmpC.channelName = chantmp;
    tmpC.watek = th;
    tmpC.color = color;
    tmpC.perm = 0;
    clientList.addElement(tmpC);
    th.login = login;
    th.channelName = chantmp;
    th.color = color;
    if (!moderatedHT.containsKey(th.channelName))
	moderatedHT.put(th.channelName, new Boolean(false));
    if ((log.equals("yes")) && (logLevel>=2)) {
	Calendar now = Calendar.getInstance();
	String ampm = "AM";
	if (now.get(Calendar.AM_PM) == 1) ampm = "PM";
	String time = now.get(Calendar.DAY_OF_MONTH) + "/" + now.get(Calendar.MONTH) + "/" + now.get(Calendar.YEAR) + " " + now.get(Calendar.HOUR) +":"+ now.get(Calendar.MINUTE) + " " +ampm+" ";
	writeToLogfile(new String(time + "JOIN " + th.login +" FROM "+th.host),chantmp);
	}
    sayToAll("*JOIN|"+login+"|",chantmp);
    return(true);
    }

/*******************************************************************************************/
/* LOGOUT FROM SYSTEM       							   */
/*******************************************************************************************/
   public  boolean quitFromSystem(String login)
 {
   if (login == null || login.equals("")) { return(false); };
   for (int i = 0; i < clientList.size(); i++) {
	kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(login)) {
	    sayToAll("*QUIT|"+login+"|",tmpE.channelName);
	    if (login.equals((String)moderatorNickHT.get(tmpE.channelName))) {
		moderatedHT.put(tmpE.channelName,new Boolean(false));
		moderatorNickHT.put(tmpE.channelName,new String(""));
		guestNickHT.put(tmpE.channelName,new String(""));
		sayToAll("*MODERATED|NO|",tmpE.channelName);
		}
	    clientList.removeElementAt(i);
	    // close connection
	    try { tmpE.watek.clientS.close(); }
catch (IOException e) { e.printStackTrace(); }
	    // stop thread
	    tmpE.watek = null;
	    }
	}
   return(true);
   }

/*******************************************************************************************/
/* MESSAGE       							   */
/*******************************************************************************************/
    public boolean sendMsgTo(String from, String to, String text, String color, String chantmp)
 {
    boolean cantalk = false;
    int poz = -1;
    kChatClass tmpE = new kChatClass();

    Boolean bval = (Boolean)moderatedHT.get(chantmp);
    if (bval.booleanValue()) {
	if (to.equals((String)guestNickHT.get(chantmp)) || to.equals((String)moderatorNickHT.get(chantmp))) return(false); // if to moderator or guest (priv) ignore
	if (to.equals("CHAN")) to = (String)moderatorNickHT.get(chantmp); // redirect all to moderator
	if (from.equals((String)guestNickHT.get(chantmp)) || from.equals((String)moderatorNickHT.get(chantmp))) to = "CHAN";  // from moder or guest redir to CHAN
	}

    // check if you want to do something as somone who doesn't exist in system
    for (int i = 0; i < clientList.size(); i++) {
	tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(from)) cantalk = true;
	if (tmpE.login.equals(to)) poz = i;
	}

    if (!cantalk) return(false);

    if (!to.equals("CHAN")) {
 // priv
	  if (poz != -1)  {
 // if "to" really exists
    	    tmpE = (kChatClass)clientList.elementAt(poz);
    	    tmpE.watek.pw.println("*MSG|"+from+"|"+to+"|"+text+"|" + color + "|\r\n");
    	    tmpE.watek.pw.flush();
    	    }

	} else { // to CHAN
	bval = (Boolean)moderatedHT.get(chantmp);
	if ((bval.booleanValue())&&(text.startsWith("~<"))&&(from.equals((String)moderatorNickHT.get(chantmp)))) {
	    from = text.substring(2,text.indexOf(" ",2)-1);
	    text = text.substring(text.indexOf(" ",2)+1,text.length());
	    }
	sayToAll("*MSG|"+from+"|"+to+"|"+text+"|" + color +"|",chantmp);
	}
    return(true);
    }

}
